@@ -1,18 +1,33 @@
-# COLMAP - Structure-from-Motion and Multi-View Stereo.
-# Copyright (C) 2017  Johannes L. Schoenberger <jsch at inf.ethz.ch>
+# Copyright (c) 2018, ETH Zurich and UNC Chapel Hill.
+# All rights reserved.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#     * Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#
+#     * Neither the name of ETH Zurich and UNC Chapel Hill nor the names of
+#       its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+#
+# Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
 
 import os
 import sys
@@ -23,6 +38,7 @@ import platform
 import argparse
 import zipfile
 import hashlib
+import ssl
 import urllib.request
 import subprocess
 import multiprocessing
@@ -59,22 +75,32 @@ def parse_args():
                         help="The path to the folder containing Boost, "
                              "e.g., under Windows: "
                              "C:/local/boost_1_64_0/lib64-msvc-12.0")
+    parser.add_argument("--cgal_path", default="",
+                        help="The path to the folder containing CGAL, "
+                             "e.g., under Windows: C:/dev/CGAL-4.11.2/build")
     parser.add_argument("--cuda_path", default="",
                         help="The path to the folder containing CUDA, "
                              "e.g., under Windows: C:/Program Files/NVIDIA GPU "
                              "Computing Toolkit/CUDA/v8.0")
-    parser.add_argument("--cuda_multi_arch",
-                        dest="cuda_multi_arch", action="store_true",)
-    parser.add_argument("--no_cuda_multi_arch",
-                        dest="cuda_multi_arch", action="store_false",
-                        help="Whether to compile CUDA code for "
-                             "multiple GPU architectures (default no)")
+    parser.add_argument("--cuda_archs", default="Auto",
+                        help="List of CUDA architectures for which to generate "
+                             "code, e.g., Auto, All, Maxwell, Pascal, ...")
     parser.add_argument("--with_suite_sparse",
                         dest="with_suite_sparse", action="store_true")
     parser.add_argument("--without_suite_sparse",
                         dest="with_suite_sparse", action="store_false",
                         help="Whether to use SuiteSparse as a sparse solver "
                              "(default with SuiteSparse)")
+    parser.add_argument("--with_cuda",
+                        dest="with_cuda", action="store_true")
+    parser.add_argument("--without_cuda",
+                        dest="with_cuda", action="store_false",
+                        help="Whether to enable CUDA functionality")
+    parser.add_argument("--with_opengl",
+                        dest="with_opengl", action="store_true")
+    parser.add_argument("--without_opengl",
+                        dest="with_opengl", action="store_false",
+                        help="Whether to enable OpenGL functionality")
     parser.add_argument("--with_tests",
                         dest="with_tests", action="store_true")
     parser.add_argument("--without_tests",
@@ -84,9 +110,17 @@ def parse_args():
                         help="Build type, e.g., Debug, Release, RelWithDebInfo")
     parser.add_argument("--cmake_generator", default="",
                         help="CMake generator, e.g., Visual Studio 14")
-    parser.set_defaults(cuda_multi_arch=False)
+    parser.add_argument("--no_ssl_verification",
+                        dest="ssl_verification", action="store_false",
+                        help="Whether to disable SSL certificate verification "
+                             "while downloading the source code")
+
     parser.set_defaults(with_suite_sparse=True)
+    parser.set_defaults(with_cuda=True)
+    parser.set_defaults(with_opengl=True)
     parser.set_defaults(with_tests=True)
+    parser.set_defaults(ssl_verification=True)
+
     args = parser.parse_args()
 
     args.build_path = os.path.abspath(args.build_path)
@@ -116,6 +150,9 @@ def parse_args():
         args.cmake_build_args.append(
             "-j{}".format(multiprocessing.cpu_count()))
 
+    if not args.ssl_verification:
+        ssl._create_default_https_context = ssl._create_unverified_context
+
     return args
 
 
@@ -181,13 +218,13 @@ def build_eigen(args):
     if os.path.exists(path):
         return
 
-    url = "https://github.com/RLovelett/eigen/archive/3.3.4.zip"
-    archive_path = os.path.join(args.download_path, "eigen.zip")
+    url = "https://bitbucket.org/eigen/eigen/get/3.3.4.zip"
+    archive_path = os.path.join(args.download_path, "eigen-3.3.4.zip")
     download_zipfile(url, archive_path, args.build_path,
-                     "5e6c210a4cd6821f6147c96fd3aab8a7")
-    shutil.move(os.path.join(args.build_path, "eigen-3.3.4"), path)
+                     "e337acc279874bc6a56da4d973a723fb")
+    shutil.move(glob.glob(os.path.join(args.build_path, "eigen-*"))[0], path)
 
-    build_cmake_project(args, os.path.join(path, "build"))
+    build_cmake_project(args, os.path.join(path, "__build__"))
 
 
 def build_freeimage(args):
@@ -198,7 +235,7 @@ def build_freeimage(args):
     if PLATFORM_IS_WINDOWS:
         url = "https://kent.dl.sourceforge.net/project/freeimage/" \
               "Binary%20Distribution/3.17.0/FreeImage3170Win32Win64.zip"
-        archive_path = os.path.join(args.download_path, "freeimage.zip")
+        archive_path = os.path.join(args.download_path, "freeimage-3.17.0.zip")
         download_zipfile(url, archive_path, args.build_path,
                          "a7e6f2f261e72260ec5b91c2a0f4bde3")
         shutil.move(os.path.join(args.build_path, "FreeImage"), path)
@@ -214,8 +251,9 @@ def build_freeimage(args):
     else:
         url = "https://kent.dl.sourceforge.net/project/freeimage/" \
               "Source%20Distribution/3.17.0/FreeImage3170.zip"
-        archive_path = os.path.join(args.download_path, "freeimage.zip")
-        download_zipfile(url, archive_path, args.build_path)
+        archive_path = os.path.join(args.download_path, "freeimage-3.17.0.zip")
+        download_zipfile(url, archive_path, args.build_path,
+                         "459e15f0ec75d6efa3c7bd63277ead86")
         shutil.move(os.path.join(args.build_path, "FreeImage"), path)
 
         if PLATFORM_IS_MAC:
@@ -265,12 +303,12 @@ def build_glew(args):
 
     url = "https://kent.dl.sourceforge.net/project/glew/" \
           "glew/2.1.0/glew-2.1.0.zip"
-    archive_path = os.path.join(args.download_path, "glew.zip")
+    archive_path = os.path.join(args.download_path, "glew-2.1.0.zip")
     download_zipfile(url, archive_path, args.build_path,
                      "dff2939fd404d054c1036cc0409d19f1")
     shutil.move(os.path.join(args.build_path, "glew-2.1.0"), path)
 
-    build_cmake_project(args, os.path.join(path, "build/cmake/build"))
+    build_cmake_project(args, os.path.join(path, "build/cmake/__build__"))
 
     if PLATFORM_IS_WINDOWS:
         shutil.move(os.path.join(args.install_path, "bin/glew32.dll"),
@@ -285,13 +323,13 @@ def build_gflags(args):
         return
 
     url = "https://github.com/gflags/gflags/archive/v2.2.1.zip"
-    archive_path = os.path.join(args.download_path, "gflags.zip")
+    archive_path = os.path.join(args.download_path, "gflags-2.2.1.zip")
     download_zipfile(url, archive_path, args.build_path,
                      "2d988ef0b50939fb50ada965dafce96b")
     shutil.move(os.path.join(args.build_path, "gflags-2.2.1"), path)
     os.remove(os.path.join(path, "BUILD"))
 
-    build_cmake_project(args, os.path.join(path, "build"))
+    build_cmake_project(args, os.path.join(path, "__build__"))
 
 
 def build_glog(args):
@@ -300,12 +338,12 @@ def build_glog(args):
         return
 
     url = "https://github.com/google/glog/archive/v0.3.5.zip"
-    archive_path = os.path.join(args.download_path, "glog.zip")
+    archive_path = os.path.join(args.download_path, "glog-0.3.5.zip")
     download_zipfile(url, archive_path, args.build_path,
                      "454766d0124951091c95bad33dafeacd")
     shutil.move(os.path.join(args.build_path, "glog-0.3.5"), path)
 
-    build_cmake_project(args, os.path.join(path, "build"))
+    build_cmake_project(args, os.path.join(path, "__build__"))
 
 
 def build_suite_sparse(args):
@@ -323,9 +361,10 @@ def build_suite_sparse(args):
     download_zipfile(url, archive_path, args.build_path,
                      "e7c27075e8e0afc9d2cf188630090946")
     shutil.move(os.path.join(args.build_path,
-                             "suitesparse-metis-for-windows-master"), path)
+                             "suitesparse-metis-for-windows-"
+                             "7bc503bfa2c4f1be9176147d36daf9e18340780a"), path)
 
-    build_cmake_project(args, os.path.join(path, "build"))
+    build_cmake_project(args, os.path.join(path, "__build__"))
 
     if PLATFORM_IS_WINDOWS:
         lapack_blas_path = os.path.join(args.install_path,
@@ -341,11 +380,11 @@ def build_ceres_solver(args):
     if os.path.exists(path):
         return
 
-    url = "https://github.com/ceres-solver/ceres-solver/archive/1.13.0.zip"
-    archive_path = os.path.join(args.download_path, "ceres-solver.zip")
+    url = "https://github.com/ceres-solver/ceres-solver/archive/1.14.0.zip"
+    archive_path = os.path.join(args.download_path, "ceres-solver-1.14.0.zip")
     download_zipfile(url, archive_path, args.build_path,
-                     "f32e5b222205ad0f5cf34624f4ea59df")
-    shutil.move(os.path.join(args.build_path, "ceres-solver-1.13.0"), path)
+                     "26b255b7a9f330bbc1def3b839724a2a")
+    shutil.move(os.path.join(args.build_path, "ceres-solver-1.14.0"), path)
 
     extra_config_args = [
         "-DBUILD_TESTING=OFF",
@@ -370,17 +409,15 @@ def build_ceres_solver(args):
     if PLATFORM_IS_WINDOWS:
         extra_config_args.append("-DCMAKE_CXX_FLAGS=/DGOOGLE_GLOG_DLL_DECL=")
 
-    build_cmake_project(args, os.path.join(path, "build"),
+    build_cmake_project(args, os.path.join(path, "__build__"),
                         extra_config_args=extra_config_args)
 
 
 def build_colmap(args):
     extra_config_args = []
     if args.qt_path != "":
-        extra_config_args.append("-DQt5Core_DIR={}".format(
-            os.path.join(args.qt_path, "lib/cmake/Qt5Core")))
-        extra_config_args.append("-DQt5OpenGL_DIR={}".format(
-            os.path.join(args.qt_path, "lib/cmake/Qt5OpenGL")))
+        extra_config_args.append("-DQt5_DIR={}".format(
+            os.path.join(args.qt_path, "lib/cmake/Qt5")))
 
     if args.boost_path != "":
         extra_config_args.append(
@@ -392,22 +429,33 @@ def build_colmap(args):
         extra_config_args.append(
             "-DCUDA_TOOLKIT_ROOT_DIR={}".format(args.cuda_path))
 
-    if args.cuda_multi_arch:
-        extra_config_args.append("-DCUDA_MULTI_ARCH=ON")
+    if args.with_cuda:
+        extra_config_args.append("-DCUDA_ENABLED=ON")
     else:
-        extra_config_args.append("-DCUDA_MULTI_ARCH=OFF")
+        extra_config_args.append("-DCUDA_ENABLED=OFF")
+
+    if args.cuda_archs:
+        extra_config_args.append("-DCUDA_ARCHS={}".format(args.cuda_archs))
+
+    if args.with_opengl:
+        extra_config_args.append("-DOPENGL_ENABLED=ON")
+    else:
+        extra_config_args.append("-DOPENGL_ENABLED=OFF")
 
     if args.with_tests:
         extra_config_args.append("-DTESTS_ENABLED=ON")
     else:
         extra_config_args.append("-DTESTS_ENABLED=OFF")
 
+    if args.cgal_path:
+        extra_config_args.append("-DCGAL_DIR={}".format(args.cgal_path))
+
     if PLATFORM_IS_WINDOWS:
         extra_config_args.append("-DCMAKE_CXX_FLAGS=/DGOOGLE_GLOG_DLL_DECL=")
 
     mkdir_if_not_exists(os.path.join(args.build_path, "colmap"))
 
-    build_cmake_project(args, os.path.join(args.build_path, "colmap/build"),
+    build_cmake_project(args, os.path.join(args.build_path, "colmap/__build__"),
                         extra_config_args=extra_config_args,
                         cmakelists_path=os.path.abspath(args.colmap_path))
 
@@ -425,10 +473,28 @@ def build_post_process(args):
                 os.path.join(args.qt_path, "bin/Qt5Widgets.dll"),
                 os.path.join(args.install_path, "lib/Qt5Widgets.dll"))
             mkdir_if_not_exists(
-                os.path.join(args.install_path, "bin/platforms"))
+                os.path.join(args.install_path, "lib/platforms"))
             copy_file_if_not_exists(
                 os.path.join(args.qt_path, "plugins/platforms/qwindows.dll"),
-                os.path.join(args.install_path, "bin/platforms/qwindows.dll"))
+                os.path.join(args.install_path, "lib/platforms/qwindows.dll"))
+        if args.with_cuda and args.cuda_path:
+            cudart_lib_path = glob.glob(os.path.join(args.cuda_path,
+                                                     "bin/cudart64_*.dll"))[0]
+            copy_file_if_not_exists(
+                cudart_lib_path,
+                os.path.join(args.install_path, "lib",
+                             os.path.basename(cudart_lib_path)))
+        if args.cgal_path:
+            gmp_lib_path = os.path.join(
+                args.cgal_path, "../auxiliary/gmp/lib/libgmp-10.dll")
+            if os.path.exists(gmp_lib_path):
+                copy_file_if_not_exists(
+                    gmp_lib_path,
+                    os.path.join(args.install_path, "lib/libgmp-10.dll"))
+            copy_file_if_not_exists(
+                os.path.join(args.cgal_path,
+                             "bin/Release/CGAL-vc140-mt-4.11.2.dll"),
+                os.path.join(args.install_path, "lib/CGAL-vc140-mt-4.11.2.dll"))
 
 
 def main():
