@@ -1,18 +1,33 @@
-// COLMAP - Structure-from-Motion and Multi-View Stereo.
-// Copyright (C) 2017  Johannes L. Schoenberger <jsch at inf.ethz.ch>
+// Copyright (c) 2018, ETH Zurich and UNC Chapel Hill.
+// All rights reserved.
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are met:
 //
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
+//     * Redistributions of source code must retain the above copyright
+//       notice, this list of conditions and the following disclaimer.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//     * Redistributions in binary form must reproduce the above copyright
+//       notice, this list of conditions and the following disclaimer in the
+//       documentation and/or other materials provided with the distribution.
+//
+//     * Neither the name of ETH Zurich and UNC Chapel Hill nor the names of
+//       its contributors may be used to endorse or promote products derived
+//       from this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
+// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+// POSSIBILITY OF SUCH DAMAGE.
+//
+// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
 
 #include "sfm/incremental_triangulator.h"
 
@@ -37,9 +52,11 @@ bool IncrementalTriangulator::Options::Check() const {
   return true;
 }
 
-IncrementalTriangulator::IncrementalTriangulator(const SceneGraph* scene_graph,
-                                                 Reconstruction* reconstruction)
-    : scene_graph_(scene_graph), reconstruction_(reconstruction) {}
+IncrementalTriangulator::IncrementalTriangulator(
+    const CorrespondenceGraph* correspondence_graph,
+    Reconstruction* reconstruction)
+    : correspondence_graph_(correspondence_graph),
+      reconstruction_(reconstruction) {}
 
 size_t IncrementalTriangulator::TriangulateImage(const Options& options,
                                                  const image_t image_id) {
@@ -150,7 +167,7 @@ size_t IncrementalTriangulator::CompleteImage(const Options& options,
     }
 
     if (options.ignore_two_view_tracks &&
-        scene_graph_->IsTwoViewObservation(image_id, point2D_idx)) {
+        correspondence_graph_->IsTwoViewObservation(image_id, point2D_idx)) {
       continue;
     }
 
@@ -326,12 +343,13 @@ size_t IncrementalTriangulator::Retriangulate(const Options& options) {
 
     // Find correspondences and perform retriangulation.
 
-    const std::vector<std::pair<point2D_t, point2D_t>> corrs =
-        scene_graph_->FindCorrespondencesBetweenImages(image_id1, image_id2);
+    const FeatureMatches& corrs =
+        correspondence_graph_->FindCorrespondencesBetweenImages(image_id1,
+                                                                image_id2);
 
     for (const auto& corr : corrs) {
-      const Point2D& point2D1 = image1.Point2D(corr.first);
-      const Point2D& point2D2 = image2.Point2D(corr.second);
+      const Point2D& point2D1 = image1.Point2D(corr.point2D_idx1);
+      const Point2D& point2D2 = image2.Point2D(corr.point2D_idx2);
 
       // Two cases are possible here: both points belong to the same 3D point
       // or to different 3D points. In the former case, there is nothing
@@ -344,7 +362,7 @@ size_t IncrementalTriangulator::Retriangulate(const Options& options) {
 
       CorrData corr_data1;
       corr_data1.image_id = image_id1;
-      corr_data1.point2D_idx = corr.first;
+      corr_data1.point2D_idx = corr.point2D_idx1;
       corr_data1.image = &image1;
       corr_data1.camera = &camera1;
       corr_data1.point2D = &point2D1;
@@ -352,7 +370,7 @@ size_t IncrementalTriangulator::Retriangulate(const Options& options) {
 
       CorrData corr_data2;
       corr_data2.image_id = image_id2;
-      corr_data2.point2D_idx = corr.second;
+      corr_data2.point2D_idx = corr.point2D_idx2;
       corr_data2.image = &image2;
       corr_data2.camera = &camera2;
       corr_data2.point2D = &point2D2;
@@ -406,16 +424,16 @@ size_t IncrementalTriangulator::Find(const Options& options,
                                      const point2D_t point2D_idx,
                                      const size_t transitivity,
                                      std::vector<CorrData>* corrs_data) {
-  const std::vector<SceneGraph::Correspondence>& corrs =
-      scene_graph_->FindTransitiveCorrespondences(image_id, point2D_idx,
-                                                  transitivity);
+  const std::vector<CorrespondenceGraph::Correspondence>& corrs =
+      correspondence_graph_->FindTransitiveCorrespondences(
+          image_id, point2D_idx, transitivity);
 
   corrs_data->clear();
   corrs_data->reserve(corrs.size());
 
   size_t num_triangulated = 0;
 
-  for (const SceneGraph::Correspondence corr : corrs) {
+  for (const CorrespondenceGraph::Correspondence corr : corrs) {
     const Image& corr_image = reconstruction_->Image(corr.image_id);
     if (!corr_image.IsRegistered()) {
       continue;
@@ -460,8 +478,8 @@ size_t IncrementalTriangulator::Create(
     return 0;
   } else if (options.ignore_two_view_tracks && create_corrs_data.size() == 2) {
     const CorrData& corr_data1 = create_corrs_data[0];
-    if (scene_graph_->IsTwoViewObservation(corr_data1.image_id,
-                                           corr_data1.point2D_idx)) {
+    if (correspondence_graph_->IsTwoViewObservation(corr_data1.image_id,
+                                                    corr_data1.point2D_idx)) {
       return 0;
     }
   }
@@ -584,9 +602,9 @@ size_t IncrementalTriangulator::Merge(const Options& options,
   const auto& point3D = reconstruction_->Point3D(point3D_id);
 
   for (const auto& track_el : point3D.Track().Elements()) {
-    const std::vector<SceneGraph::Correspondence>& corrs =
-        scene_graph_->FindCorrespondences(track_el.image_id,
-                                          track_el.point2D_idx);
+    const std::vector<CorrespondenceGraph::Correspondence>& corrs =
+        correspondence_graph_->FindCorrespondences(track_el.image_id,
+                                                   track_el.point2D_idx);
 
     for (const auto corr : corrs) {
       const auto& image = reconstruction_->Image(corr.image_id);
@@ -690,9 +708,9 @@ size_t IncrementalTriangulator::Complete(const Options& options,
     queue.clear();
 
     for (const TrackElement queue_elem : prev_queue) {
-      const std::vector<SceneGraph::Correspondence>& corrs =
-          scene_graph_->FindCorrespondences(queue_elem.image_id,
-                                            queue_elem.point2D_idx);
+      const std::vector<CorrespondenceGraph::Correspondence>& corrs =
+          correspondence_graph_->FindCorrespondences(queue_elem.image_id,
+                                                     queue_elem.point2D_idx);
 
       for (const auto corr : corrs) {
         const Image& image = reconstruction_->Image(corr.image_id);
